/****************************************************************************
 *																			*
 *  Copyright (c) 1999-2000 FGL Graphics, a division of SONICblue Inc.   	*
 *  All Rights Reserved.													*
 *																			*
 *  This software and its associated documentation contains proprietary,	*
 *  confidential and trade secret information of FGL Graphics,				*
 *  a division of SONICblue Inc.											*
 *																			*
 *  a)  no part may be disclosed, distributed, reproduced, transmitted,		*
 *      transcribed, stored in a retrieval system, adapted or translated	*
 *      in any form or by any means electronic, mechanical, magnetic,		*
 *      optical, chemical, manual or otherwise,								*
 *																			*
 *  and																		*
 *																			*
 *  b)  the recipient is not entitled to discover through reverse			*
 *      engineering or reverse compiling or other such techniques or		*
 *      processes the trade secrets contained therein or in the				*
 *		documentation.														*
 *																			*
 ****************************************************************************/

#ifndef _FIREGL_PUBLIC_H_
#define _FIREGL_PUBLIC_H_

#define FGL_DEVICE_SIGNATURE    0x10020000

#define __KE_NO_VSPRINTF

/*****************************************************************************/

struct inode;
struct file;
struct vm_area_struct;
struct wait_queue;
struct semaphore;
struct _agp_memory;
struct mm_struct;
struct pci_dev;
struct pci_driver;
struct pci_bus;

/*****************************************************************************/

typedef unsigned int __ke_dev_t;
typedef unsigned int __ke_size_t;
typedef unsigned long __ke_off_t;
typedef int __ke_ssize_t;
typedef unsigned char __ke_u8;
typedef unsigned short __ke_u16;
typedef unsigned int __ke_u32;
typedef void* __ke_wait_queue_t;
typedef void* __ke_sigset_t;
typedef void* __ke_pci_dev_t;
typedef	int (*__ke_read_proc_t)(char* page, char** start, __ke_off_t off, int count, int* eof, void* data);
typedef struct {
    const char*         name;
    __ke_read_proc_t    f;
} __ke_proc_list_t;
typedef void *__ke_priv_device_t;
typedef struct {
    unsigned long       signature;
    __ke_priv_device_t  privdev;
    __ke_proc_list_t *  proclist;
    const char *        name;
	unsigned int	    major_version;
	unsigned int	    minor_version;
	unsigned int        patchlevel;
	const char *	    date;
} __ke_device, *__ke_device_t;

typedef struct __ke_pci_device_info
{
    unsigned short	vendor;
	unsigned short	device;
	unsigned short	subsystem_vendor;
	unsigned short	subsystem_device;
} __ke_pci_device_info_t;

typedef struct __ke_list_head
{
	struct __ke_list_head *next;
	struct __ke_list_head *prev;
} __ke_list_head_t;

/*****************************************************************************/

extern int firegl_init(__ke_device_t);
extern void firegl_cleanup(__ke_device_t);
extern int firegl_open(struct inode* inode, struct file* filp);
extern int firegl_release(struct inode* inode, struct file* filp);
extern int firegl_ioctl(struct inode* inode,
                        struct file* filp,
                        unsigned int cmd,
                        unsigned long arg);

extern void firegl_vm_bq_open(struct vm_area_struct* vma);
extern void firegl_vm_bq_close(struct vm_area_struct* vma);

/*****************************************************************************/

extern int firegl_mmap(struct file* filp, struct vm_area_struct* vma);
extern void drm_vm_open(struct vm_area_struct* vma);
extern void drm_vm_close(struct vm_area_struct* vma);
extern void drm_vm_ctx_open(struct vm_area_struct* vma);
extern void drm_vm_ctx_close(struct vm_area_struct* vma);

/*****************************************************************************/

extern void __ke_init_waitqueue_head(__ke_wait_queue_t* queue);
extern void __ke_wake_up_interruptible(__ke_wait_queue_t* queue);
extern void __ke_add_wait_queue(__ke_wait_queue_t* queue, __ke_wait_queue_t entry);
extern void __ke_remove_wait_queue(__ke_wait_queue_t* queue, __ke_wait_queue_t entry);
extern void __ke_schedule(void);
extern int __ke_signal_pending(void);

extern void __ke_set_current_state_task_interruptible(void);
extern void __ke_set_current_state_task_running(void);
extern void __ke_configure_sigmask(__ke_sigset_t *pSigMask);
extern void __ke_block_all_signals(int (*notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask);
extern void __ke_unblock_all_signals(void);

extern unsigned long __ke__cmpxchg(volatile void *ptr, unsigned long old,                      
                      unsigned long new, int size);

#define __ke_cmpxchg(ptr,o,n)                        \
  ((__typeof__(*(ptr)))__ke__cmpxchg((ptr),(unsigned long)(o),      \
                 (unsigned long)(n),sizeof(*(ptr))))
                                                                                        
/*****************************************************************************/

extern __ke_dev_t __ke_getdevice(__ke_device_t dev);
extern const char* __ke_module_parm(void);
extern int __ke_inode_rdev_minor(struct inode* inode);
extern void* __ke_get_file_priv(struct file* filp);
extern void __ke_set_file_priv(struct file* filp, void* private_data);
extern int __ke_file_excl_open(struct file* filp);
extern int __ke_file_rw_open(struct file* filp);
extern unsigned int __ke_file_counter(struct file* filp);
extern int __ke_getpid(void);
extern int __ke_geteuid(void);
extern unsigned long __ke_jiffies(void);
extern void __ke_udelay(unsigned long usecs);
extern void __ke_mdelay(unsigned long msecs);
extern unsigned long __ke_virt_to_bus(void* address);
extern unsigned long __ke_virt_to_phys(void* address);
extern void* __ke_high_memory(void);

enum __ke_error_num
{
    __KE_EBUSY,
    __KE_EINVAL,
    __KE_EACCES,
    __KE_EFAULT,
    __KE_EIO,
    __KE_EBADSLT,
    __KE_ENOMEM,
    __KE_EPERM,
    __KE_ENODEV,
    __KE_EINTR,
    __KE_ERESTARTSYS,
    __KE_ELIBBAD,
};
extern int __ke_error_code(enum __ke_error_num errcode);

extern void __ke_mod_inc_use_count(void);
extern void __ke_mod_dec_use_count(void);
extern void __ke_down_struct_sem(__ke_device_t dev, int index);
extern void __ke_up_struct_sem(__ke_device_t dev, int index);
	#define __KE_MAX_SEMAPHORES 2
extern void __ke_sema_init(struct semaphore* sem, int value);
extern __ke_size_t __ke_sema_size(void);
extern void __ke_down(struct semaphore* sem);
extern void __ke_up(struct semaphore* sem);
extern void __ke_atomic_inc(void* v);
extern void __ke_atomic_dec(void* v);
extern void __ke_atomic_add(int val, void* v);
extern void __ke_atomic_sub(int val, void* v);
extern int __ke_atomic_read(void* v);
extern void __ke_atomic_set(void* v, int val);
extern void __ke_spin_lock(__ke_device_t dev, int ndx);
extern void __ke_spin_unlock(__ke_device_t dev, int ndx);
	#define __KE_MAX_SPINLOCKS 17
extern void __ke_lock_kernel(void);
extern void __ke_unlock_kernel(void);
extern int __ke_sys_mlock(unsigned long start, __ke_size_t len);
extern int __ke_sys_munlock(unsigned long start, __ke_size_t len);
#ifdef __KE_NO_VSPRINTF
extern void __ke_printk(const char* fmt, ...);
#else // !__KE_NO_VSPRINTF
extern void __ke_print_info(const char* fmt, ...);
extern void __ke_print_error(const char* fmt, ...);
extern void __ke_print_debug(const char* fmt, ...);
#endif // !__KE_NO_VSPRINTF

enum __ke_cap
{
    __KE_CAP_SYS_ADMIN,
    __KE_CAP_IPC_LOCK,
};
extern int __ke_capable(enum __ke_cap cap);
extern void __ke_cap_effective_raise(enum __ke_cap cap);
extern __ke_u32 __ke_get_cap_effective(void);
extern void __ke_set_cap_effective(__ke_u32 cap);
extern unsigned long __ke_ram_available(void);

extern int __ke_copy_from_user(void* to, const void* from, __ke_size_t size);
extern int __ke_copy_to_user(void* to, const void* from, __ke_size_t size);
extern int __ke_verify_area(int type, const void * addr, unsigned long size);

extern void* __ke_malloc(__ke_size_t size);
extern void __ke_free_s(void* p, __ke_size_t size);
extern void* __ke_vmalloc(__ke_size_t size);
extern void __ke_vfree(void* p);
extern void* __ke_get_free_page(void);
extern void* __ke_get_free_pages(int order);
extern void __ke_free_page(void* pt);
extern void __ke_free_pages(void* pt, int order);
extern void __ke_mem_map_reserve(void* pt);
extern void __ke_mem_map_unreserve(void* pt);
extern void __ke_virt_reserve(void* pt);
extern void __ke_virt_unreserve(void* pt);
extern void* __ke_ioremap(unsigned long offset, unsigned long size);
extern void* __ke_ioremap_nocache(unsigned long offset, unsigned long size);
extern void __ke_iounmap(void* pt);
extern int __ke_verify_read_access(void* addr, __ke_size_t size);
extern int __ke_verify_write_access(void* addr, __ke_size_t size);
extern struct mm_struct* __ke_init_mm(void);
extern struct mm_struct* __ke_current_mm(void);
extern unsigned long __ke_get_phys_addr(struct mm_struct* mm, unsigned long virtual_addr);
extern void* __ke_memset(void* s, int c, __ke_size_t count);
extern void* __ke_memcpy(void* d, const void* s, __ke_size_t count);
extern __ke_size_t __ke_strlen(const char *s);
extern char* __ke_strcpy(char* d, const char* s);
extern char* __ke_strncpy(char* d, const char* s, __ke_size_t count);
extern int __ke_strcmp(const char *s1, const char *s2);
extern int __ke_strncmp(const char* str1, const char* str2, __ke_size_t count);
extern int __ke_sprintf(char* buf, const char* fmt, ...);

/*****************************************************************************/

extern void __ke_set_bit(int nr, volatile void * addr);
extern void __ke_clear_bit(int nr, volatile void * addr);

/*****************************************************************************/

extern int __ke_flush_cache(void);

/*****************************************************************************/

extern int __ke_config_mtrr(void);
extern int __ke_mtrr_add_wc(unsigned long base, unsigned long size);
extern int __ke_mtrr_del(int reg, unsigned long base, unsigned long size);

/*****************************************************************************/
extern int __ke_pci_get_irq(struct pci_dev *dev, unsigned int* irq);
extern int __ke_get_pci_device_info(struct pci_dev *dev, __ke_pci_device_info_t *pinfo);
extern int __ke_check_pci(int busnum, int devnum, int funcnum, __ke_u16* vendor, __ke_u16* device, unsigned int* irq);
extern struct pci_dev *__ke_pci_find_device (unsigned int vendor, unsigned int dev, struct pci_dev *from);
extern int __ke_pci_read_config_byte(struct pci_dev *dev, __ke_u8 where, __ke_u8 *val);
extern int __ke_pci_read_config_word(struct pci_dev *dev, __ke_u8 where, __ke_u16 *val);
extern int __ke_pci_read_config_dword(struct pci_dev *dev, __ke_u8 where, __ke_u32 *val);
extern int __ke_pci_write_config_byte(struct pci_dev *dev, __ke_u8 where, __ke_u8 val);
extern int __ke_pci_write_config_word(struct pci_dev *dev, __ke_u8 where, __ke_u16 val);
extern int __ke_pci_write_config_dword(struct pci_dev *dev, __ke_u8 where, __ke_u32 val);

/*****************************************************************************/

extern void __ke_outb(unsigned char value, unsigned short port);
extern void __ke_outw(unsigned short value, unsigned short port);
extern void __ke_out(unsigned int value, unsigned short port);
extern char __ke_inb(unsigned short port);
extern short __ke_inw(unsigned short port);
extern int __ke_in(unsigned short port);

/*****************************************************************************/

extern void __ke_dbg_irq( void );
extern void __ke_enable_irq( int irq );
extern void __ke_disable_irq( int irq );
extern int __ke_request_irq( unsigned int irq, void (*handler)(int, void *, void *), const char *dev_name, void *dev_id);
extern void __ke_free_irq( unsigned int irq, void *dev_id );
extern int __ke_queue_task( void *task, void *data );
extern void __ke_mark_bh( void );


/*****************************************************************************/


/*****************************************************************************/


extern void* __ke_vma_file_priv(struct vm_area_struct* vma);
extern unsigned long __ke_vm_start(struct vm_area_struct* vma);
extern unsigned long __ke_vm_end(struct vm_area_struct* vma);
extern unsigned long __ke_vm_offset(struct vm_area_struct* vma);
enum __ke_vm_maptype
{
    __KE_ADPT,
    __KE_SHM,
    __KE_CTX,
    __KE_PCI_BQS,
    __KE_AGP_BQS,
    __KE_AGP,
};
extern char* __ke_vm_flags_str(struct vm_area_struct* vma, char* buf);
extern char* __ke_vm_page_prot_str(struct vm_area_struct* vma, char* buf);
extern char* __ke_vm_phys_addr_str(struct vm_area_struct* vma, 
                                   char* buf, 
                                   unsigned long linear_address, 
                                   unsigned long* phys_address);
extern int __ke_vm_map(struct file* filp,
                       struct vm_area_struct* vma,
                       enum __ke_vm_maptype type,
                       int readonly);

/*****************************************************************************/

typedef struct __ke_agp_version
{
    __ke_u16 major;
    __ke_u16 minor;
} __ke_agp_version_t;

typedef struct __ke_agp_kern_info
{
    __ke_agp_version_t version;
    __ke_u16 vendor;
    __ke_u16 device;
    unsigned long mode;
    __ke_off_t aper_base;
    __ke_size_t aper_size;
    int max_memory;     /* In pages */
    int current_memory;
} __ke_agp_kern_info_t;

/*****************************************************************************/

extern int __ke_agp_try_unsupported;

int __ke_agp_available(void);
void __ke_agp_uninit(void);
void __ke_agp_free_memory(struct _agp_memory* handle);
struct _agp_memory* __ke_agp_allocate_memory(__ke_size_t pages, 
                                             unsigned long type);
int __ke_agp_bind_memory(struct _agp_memory* handle, __ke_off_t start);
int __ke_agp_unbind_memory(struct _agp_memory* handle);
int __ke_agp_enable(unsigned long mode);
int __ke_read_agp_command_register(struct pci_dev *pdevice, unsigned long *cmd);
int __ke_agp_acquire(void);
void __ke_agp_release(void);
void __ke_agp_copy_info(__ke_agp_kern_info_t* info);
unsigned long __ke_agp_memory_handle(struct _agp_memory* handle);
unsigned long __ke_agp_memory_page_count(struct _agp_memory* handle);

/*****************************************************************************/

struct _agp_memory* firegl_agp_allocate_memory(__ke_size_t page_count, __ke_u32 type);
void firegl_agp_free_memory(struct _agp_memory* curr);
int firegl_agp_bind_memory(struct _agp_memory* curr, __ke_off_t pg_start);
int firegl_agp_unbind_memory(struct _agp_memory* curr);
void firegl_agp_enable(__ke_u32 mode);
int firegl_agp_acquire(void);
void firegl_agp_release(void);
void firegl_agp_copy_info(__ke_agp_kern_info_t* info);
int firegl_agp_init(void);
void firegl_agp_cleanup(void);
unsigned long firegl_agp_memory_handle(struct _agp_memory* handle);
unsigned long firegl_agp_memory_page_count(struct _agp_memory* handle);

int __ke_is_athlon(void);
void __ke_smp_call_function( void (*func)(void *info) );
int __ke_smp_processor_id(void);

/*****************************************************************************/

/* globals */
extern int __ke_debflags;
extern int __ke_debuglevel;
extern int __ke_moduleflags;

/* module flags */
#define __KE_FLAG_DEBUG                     0x00000001
#define __KE_FLAG_NOAUTH                    0x00000002
#define __KE_FLAG_DISABLE_AGPGART           0x00000004
#define __KE_FLAG_DISABLE_FIREGL_AGPGART    0x00000008

/* global constants */
extern const unsigned long __ke_HZ;
extern const char*         __ke_UTS_RELEASE;
extern const unsigned long __ke_LINUX_VERSION_CODE;
extern const unsigned int  __ke_PAGE_SHIFT;
extern const unsigned int  __ke_PAGE_SIZE;
extern const unsigned int  __ke_PAGE_MASK;

extern const unsigned long __ke_MODVERSIONS_State;
extern const unsigned long __ke_SMP_State;

/*****************************************************************************/

#ifdef __KE_NO_VSPRINTF
#define	__KE_KERN_ERR	"<3>"
#define	__KE_KERN_INFO	"<6>"
#define	__KE_KERN_DEBUG	"<7>"

                /* Macros to make printk easier */
#define __KE_ERROR(fmt, arg...)                                             \
    __ke_printk(__KE_KERN_ERR "[" FIREGL_KERNEL_DRIVER_NAME ":"             \
        __FUNCTION__ "] *ERROR* " fmt , ##arg)

#define __KE_INFO(fmt, arg...)                                              \
    __ke_printk(__KE_KERN_INFO "[" FIREGL_KERNEL_DRIVER_NAME "] " fmt , ##arg)

#if 1 /* Leave this on to be able to debug in case of any problems */
#define __KE_DEBUG0(fmt, arg...)                                            \
    do                                                                      \
    {                                                                       \
        __ke_printk(__KE_KERN_DEBUG "[" FIREGL_KERNEL_DRIVER_NAME ":"		\
            __FUNCTION__ "] " fmt , ##arg);									\
    } while (0)
#define __KE_DEBUG(fmt, arg...)                                             \
    do                                                                      \
    {                                                                       \
        if (__ke_debuglevel > 0)                                            \
            __ke_printk(__KE_KERN_DEBUG "[" FIREGL_KERNEL_DRIVER_NAME ":"   \
                __FUNCTION__ "] " fmt , ##arg);                             \
    } while (0)
#define __KE_DEBUG2(fmt, arg...)                                            \
    do                                                                      \
    {                                                                       \
        if (__ke_debuglevel > 1)                                            \
            __ke_printk(__KE_KERN_DEBUG "[" FIREGL_KERNEL_DRIVER_NAME ":"   \
                __FUNCTION__ "] " fmt , ##arg);                             \
    } while (0)
#define __KE_DEBUG3(fmt, arg...)                                            \
    do                                                                      \
    {                                                                       \
        if (__ke_debuglevel > 2)                                            \
            __ke_printk(__KE_KERN_DEBUG "[" FIREGL_KERNEL_DRIVER_NAME ":"   \
                __FUNCTION__ "] " fmt , ##arg);                             \
    } while (0)
#define __KE_VMDEBUG(fmt, arg...)                                            \
    do                                                                      \
    {                                                                       \
        if (__ke_debflags )                                            \
            __ke_printk(__KE_KERN_DEBUG "[" FIREGL_KERNEL_DRIVER_NAME ":"   \
                __FUNCTION__ "] " fmt , ##arg);                             \
    } while (0)



#else
#define __KE_DEBUG0(fmt, arg...)    do { } while (0)
#define __KE_DEBUG(fmt, arg...)     do { } while (0)
#define __KE_DEBUG2(fmt, arg...)    do { } while (0)
#define __KE_DEBUG3(fmt, arg...)    do { } while (0)
#endif
#else // !__KE_NO_VSPRINTF
                /* Macros to make printk easier */
#define __KE_ERROR(fmt, arg...)                                             \
    __ke_print_error("[" FIREGL_KERNEL_DRIVER_NAME ":"                      \
        __FUNCTION__ "] *ERROR* " fmt , ##arg)

#define __KE_INFO(fmt, arg...)                                              \
    __ke_print_info("[" FIREGL_KERNEL_DRIVER_NAME "] " fmt , ##arg)

#if 1 /* Leave this on to be able to debug in case of any problems */
#define __KE_DEBUG(fmt, arg...)                                             \
    do                                                                      \
    {                                                                       \
        if (__ke_debuglevel > 0)                                            \
            __ke_print_debug("[" FIREGL_KERNEL_DRIVER_NAME ":"              \
                __FUNCTION__ "] " fmt , ##arg);                             \
    } while (0)
#define __KE_DEBUG2(fmt, arg...)                                            \
    do                                                                      \
    {                                                                       \
        if (__ke_debuglevel > 1)                                            \
            __ke_print_debug("[" FIREGL_KERNEL_DRIVER_NAME ":"              \
                __FUNCTION__ "] " fmt , ##arg);                             \
    } while (0)
#define __KE_DEBUG3(fmt, arg...)                                            \
    do                                                                      \
    {                                                                       \
        if (__ke_debuglevel > 2)                                            \
            __ke_print_debug("[" FIREGL_KERNEL_DRIVER_NAME ":"              \
                __FUNCTION__ "] " fmt , ##arg);                             \
    } while (0)
#else
#define __KE_DEBUG(fmt, arg...)     do { } while (0)
#define __KE_DEBUG2(fmt, arg...)    do { } while (0)
#define __KE_DEBUG3(fmt, arg...)    do { } while (0)
#endif
#endif // !__KE_NO_VSPRINTF

/*****************************************************************************/

#endif /* _FIREGL_PUBLIC_H_ */
